"""Manim Recorder plugin for manim"""

from manim_recorder.voiceover_scene import RecorderScene
from manim_recorder.tracker import SoundTracker

# import pkg_resources

# __version__: str = pkg_resources.get_distribution(__name__).version

# __version__ = "0.0.1"
